@@ -8,12 +8,11 @@ from dataset.persistence.util import sqlite_datetime_fix
 from dataset.persistence.database import Database
 from dataset.persistence.table import Table
 from dataset.freeze.app import freeze
-from sqlalchemy import Integer, Text
 
 __all__ = ['Database', 'Table', 'freeze', 'connect']
 
 
-def connect(url=None, schema=None, reflectMetadata=True):
+def connect(url=None, schema=None, reflectMetadata=True, engine_kwargs=None):
     """
     Opens a new connection to a database. *url* can be any valid `SQLAlchemy engine URL`_.
     If *url* is not defined it will try to use *DATABASE_URL* from environment variable.
@@ -32,4 +31,5 @@ def connect(url=None, schema=None, reflectMetadata=True):
     if url.startswith("sqlite://"):
         sqlite_datetime_fix()
 
-    return Database(url, schema=schema, reflectMetadata=reflectMetadata)
+    return Database(url, schema=schema, reflectMetadata=reflectMetadata,
+                    engine_kwargs=engine_kwargs)
