@@ -1,13 +1,9 @@
 import logging
 from itertools import count
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict  # Python < 2.7 drop-in
 
 from sqlalchemy.sql import and_, expression
 from sqlalchemy.schema import Column, Index
-
+from sqlalchemy import alias
 from dataset.persistence.util import guess_type
 from dataset.persistence.util import ResultIter
 from dataset.util import DatasetException
@@ -28,11 +24,8 @@ class Table(object):
     def columns(self):
         """
         Get a listing of all columns that exist in the table.
-
-        >>> print 'age' in table.columns
-        True
         """
-        return set(self.table.columns.keys())
+        return list(self.table.columns.keys())
 
     def drop(self):
         """
@@ -94,7 +87,7 @@ class Table(object):
                     self._ensure_columns(row, types=types)
             self.table.insert().execute(chunk)
         self._check_dropped()
-        
+
         chunk = []
         for i, row in enumerate(rows, start=1):
             chunk.append(row)
@@ -104,7 +97,6 @@ class Table(object):
 
         if chunk:
             _process_chunk(chunk)
-        
 
     def update(self, row, keys, ensure=True, types={}):
         """
@@ -126,7 +118,7 @@ class Table(object):
         if not isinstance(keys, (list, tuple)):
             keys = [keys]
         self._check_dropped()
-        if not keys or len(keys)==len(row):
+        if not keys or len(keys) == len(row):
             return False
         clause = [(u, row.get(u)) for u in keys]
 
@@ -189,10 +181,14 @@ class Table(object):
             stmt = self.table.delete(q)
         else:
             stmt = self.table.delete()
-        self.database.executable.execute(stmt)
+        rows = self.database.executable.execute(stmt)
+        return rows.rowcount > 0
 
     def _ensure_columns(self, row, types={}):
-        for column in set(row.keys()) - set(self.table.columns.keys()):
+        # Keep order of inserted columns
+        for column in row.keys():
+            if column in self.table.columns.keys():
+                continue
             if column in types:
                 _type = types[column]
             else:
@@ -288,7 +284,7 @@ class Table(object):
         rp = self.database.executable.execute(query)
         data = rp.fetchone()
         if data is not None:
-            return OrderedDict(zip(rp.keys(), data))
+            return data
 
     def _args_to_order_by(self, order_by):
         if order_by[0] == '-':
@@ -333,10 +329,13 @@ class Table(object):
         args = self._args_to_clause(_filter)
 
         # query total number of rows first
-        count_query = self.table.count(whereclause=args, limit=_limit, offset=_offset)
+        count_query = alias(self.table.select(whereclause=args, limit=_limit, offset=_offset), name='count_query_alias').count()
         rp = self.database.executable.execute(count_query)
         total_row_count = rp.fetchone()[0]
 
+        if _limit is None:
+            _limit = total_row_count
+
         if _step is None or _step is False or _step == 0:
             _step = total_row_count
 
@@ -348,13 +347,9 @@ class Table(object):
 
         for i in count():
             qoffset = _offset + (_step * i)
-            qlimit = _step
-            if _limit is not None:
-                qlimit = min(_limit - (_step * i), _step)
+            qlimit = min(_limit - (_step * i), _step)
             if qlimit <= 0:
                 break
-            if qoffset > total_row_count:
-                break
             queries.append(self.table.select(whereclause=args, limit=qlimit,
                                              offset=qoffset, order_by=order_by))
         return ResultIter((self.database.executable.execute(q) for q in queries))
@@ -409,6 +404,9 @@ class Table(object):
         ::
 
             for row in table:
-                print row
+                print(row)
         """
         return self.all()
+
+    def __repr__(self):
+        return '<Table(%s)>' % self.table.name
